package com.mycompany.myfirstemptybot;

import java.util.logging.Level;

import com.mycompany.myfirstemptybot.nmnav.NavMeshNavigation;

import cz.cuni.amis.pogamut.base.agent.navigation.IPathExecutorState;
import cz.cuni.amis.pogamut.base.agent.navigation.PathExecutorState;
import cz.cuni.amis.pogamut.base.utils.guice.AgentScoped;
import cz.cuni.amis.pogamut.ut2004.agent.module.sensor.NavigationGraphBuilder;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.IUT2004Navigation;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.NavigationState;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.UT2004MapTweaks.IMapTweak;
import cz.cuni.amis.pogamut.ut2004.agent.navigation.UT2004PathExecutorStuckState;
import cz.cuni.amis.pogamut.ut2004.bot.impl.UT2004Bot;
import cz.cuni.amis.pogamut.ut2004.bot.impl.UT2004BotModuleController;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbcommands.Initialize;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbcommands.Respawn;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.BotKilled;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.ConfigChange;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.GameInfo;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.InitedMessage;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.NavPoint;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.NavPointNeighbourLink;
import cz.cuni.amis.pogamut.ut2004.communication.messages.gbinfomessages.Self;
import cz.cuni.amis.pogamut.ut2004.utils.UT2004BotRunner;
import cz.cuni.amis.utils.exception.PogamutException;
import cz.cuni.amis.utils.flag.FlagListener;

@AgentScoped
public class NavBot extends UT2004BotModuleController<UT2004Bot> {

	private boolean shouldReloadNavMesh = true;
    private boolean shouldUseNMNav = true;
    private boolean shouldDrawNavMesh = false;
	
    private IUT2004Navigation myNav;
	
    @Override
    public Initialize getInitializeCommand() {
        return new Initialize().setName("NavBot");        
    }

    @Override
    public void mapInfoObtained() {
        super.mapInfoObtained();
        
        mapTweaks.register("DM-1on1-Roughinery-FPS", new IMapTweak() {
			@Override
			public void tweak(NavigationGraphBuilder builder) {
				// Place navBuilder code in-here
				
				// EXAMPLE: How to change "z" coordinate of the NavPoint
				//builder.modifyNavPoint("PathNode80").addZ(-50).apply();
				
				// EXAMPLE: How to remove edge going from PathNode80 to PathNode81
				//builder.removeEdge("PathNode80", "PathNode81");
				
				// EXAMPLE: How to remove any edge that links nodes PathNode80 to PathNode81
				//builder.removeEdgesBetween("PathNode80", "PathNode81");
			}
        });
        
        // IF YOU DO ADD NEW CHANGES ABOVE
        // => always reload navmesh (reprocess / recombine with NavGraph)
        navMeshModule.setReloadNavMesh(shouldReloadNavMesh);
    }

    @Override
    public void botInitialized(GameInfo gameInfo, ConfigChange currentConfig, InitedMessage init) {
    	
    	myNav = new NavMeshNavigation(bot, info, move, navMeshModule);
    	
    	if (navMeshModule.isInitialized()) {
    		if (shouldUseNMNav) {
    			this.navigation = myNav;
    		}
    		if (shouldDrawNavMesh) {
    			navMeshModule.getNavMeshDraw().draw(true, true);
    		}
    	}
        
        navigation.addStrongNavigationListener(
            new FlagListener<NavigationState>() {
                @Override
                public void flagChanged(NavigationState changedValue) {
                    navigationStateChanged(changedValue);
                }
            }
        );  
        
        navigation.getPathExecutor().getState().addStrongListener(
            new FlagListener<IPathExecutorState>() {

                @Override
                public void flagChanged(IPathExecutorState changedValue) {                        
                    if (changedValue.getState() == PathExecutorState.STUCK) {
                        pathExecutorStuck((UT2004PathExecutorStuckState)changedValue);
                    }
                }
        
            }
        );
        
        bot.getLogger().getCategory("Yylex").setLevel(Level.OFF);      
        
    }
    
    private void navigationStateChanged(NavigationState changedValue) {
        body.getCommunication().sendGlobalTextMessage(changedValue.toString());
    }
    
    private void pathExecutorStuck(UT2004PathExecutorStuckState state) {
        body.getCommunication().sendGlobalTextMessage(state.getStuckDetector().getClass().getSimpleName());
        if (state.getLink() != null) {
        	NavPointNeighbourLink link = state.getLink();
        	body.getCommunication().sendGlobalTextMessage(link.getFromNavPoint().getId().getStringId() + " -> " + link.getToNavPoint().getId().getStringId());
        }
        // MIGHT BE USEFUL?
        // TODO: uncomment
        //System.exit(1);
    }

    @Override
    public void botFirstSpawn(GameInfo gameInfo, ConfigChange config, InitedMessage init, Self self) {
        body.getCommunication().sendGlobalTextMessage("NavBot reportin' in!");
        
        // WARNING: primary testing & validation testing should be done with Level.OFF!
        navigation.setLogLevel(Level.ALL);
    }
    
    @Override
    public void botKilled(BotKilled event) {
       
    }
    
    int state = 0;
    
    @Override
    public void logic() throws PogamutException {
        log.info("---[ LOGIC ]---");
        
        NavPoint from = navPoints.getNavPoint("PathNode68");
        NavPoint to   = navPoints.getNavPoint("PathNode77");
        
        if (state == 0) {
            // SPAWN TO "FROM"
            act.act(new Respawn().setStartLocation(from.getLocation()));
            state = 1;
            return;
        } 
        if (state == 1) {
            if (info.atLocation(from)) {
                state = 2;
            } else {
                return; 
            }
        }
        if (state == 2) {
            if (navigation.isNavigating()) return;
            if (info.atLocation(to)) {
                state = 0;
                return;
            }
            // WARNING: if navigation stuck, this will restart it!
            //          Be sure that you "kill" the bot if stuck happens!
            navigation.navigate(to);
        }
              
    }

   
   @SuppressWarnings({ "rawtypes", "unchecked" })
   public static void main(String args[]) throws PogamutException {
        new UT2004BotRunner(     // class that wraps logic for bots executions, suitable to run single bot in single JVM
                NavBot.class,    // which UT2004BotController it should instantiate
                "NavBot"         // what name the runner should be using
        ).setMain(true)          // tells runner that is is executed inside MAIN method, thus it may block the thread and watch whether agent/s are correctly executed
         .startAgents(1);        // tells the runner to start 1 agent
    }
}
